#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "StrFunc.h"
#include "Resources\resource.h"


//Include string functions
#define xmemset
#define xstrlenW
#define xstrcmpnW
#define xstrstrW
#include "StrFunc.h"

//Functions prototypes
void Exec(wchar_t *wszCmdLine, wchar_t *wszExeDir, BOOL bWaitForIdle);
HWND FindFirefoxWindow();
BOOL CALLBACK EnumWindowProc(HWND hWnd, LPARAM lParam);
DWORD GetFileSizeByName(const wchar_t *wpFile);
int GetExeDir(HINSTANCE hInstance, wchar_t *wszExeDir, int nLen);
wchar_t* GetCommandLineParamsW();

//Global variables
wchar_t wszCmdLine[1024];
wchar_t wszExeFile[MAX_PATH];
wchar_t wszExeDir[MAX_PATH];
wchar_t wszOriginalExe[MAX_PATH];
wchar_t wszTargetExe[MAX_PATH];
wchar_t wszWindow[MAX_PATH];
HINSTANCE hInstance;
HANDLE hMutex;
wchar_t *wpCommandLineParams;

//GCC
int main()
{
  return 0;
}

//Entry point
void _WinMain()
{
  if (hMutex=CreateEventW(NULL, FALSE, FALSE, L"FireFoxPortableMutex"))
  {
    if (GetLastError() != ERROR_ALREADY_EXISTS)
    {
      BOOL bOriginalExist;

      //Get program HINSTANCE
      hInstance=GetModuleHandleW(NULL);

      //Get program file and directory
      GetModuleFileNameW(hInstance, wszExeFile, MAX_PATH);
      GetExeDir(hInstance, wszExeDir, MAX_PATH);

      //Get target exe
      wsprintfW(wszOriginalExe, L"%s\\firefox-original.exe", wszExeDir);
      bOriginalExist=(GetFileAttributesW(wszOriginalExe) != (DWORD)-1);
      wsprintfW(wszTargetExe, L"%s\\firefox.exe", wszExeDir);
      if (bOriginalExist && lstrcmpiW(wszTargetExe, wszExeFile) && GetFileSizeByName(wszTargetExe) != GetFileSizeByName(wszExeFile))
      {
        //Reinstall starter after FireFox update
        if (GetFileAttributesW(wszTargetExe) != (DWORD)-1)
        {
          DeleteFileW(wszOriginalExe);
          MoveFileW(wszTargetExe, wszOriginalExe);
        }
        CopyFileW(wszExeFile, wszTargetExe, FALSE);
      }
      if (bOriginalExist)
        lstrcpynW(wszTargetExe, wszOriginalExe, MAX_PATH);

      if (GetFileAttributesW(wszTargetExe) != (DWORD)-1 && lstrcmpiW(wszTargetExe, wszExeFile))
      {
        wpCommandLineParams=GetCommandLineParamsW();

        if (xstrcmpnW(L"$$$", wpCommandLineParams, -1))
        {
          //Self call
          wsprintfW(wszCmdLine, L"\"%s\" $$$%s", wszExeFile, wpCommandLineParams);
          Exec(wszCmdLine, wszExeDir, FALSE);
        }
        else
        {
          //Find FireFox window
          HWND hWndFireFox=FindFirefoxWindow();

          //Run FireFox
          if (!hWndFireFox)
          {
            wsprintfW(wszCmdLine, L"\"%s\" -Profile profile", wszTargetExe);
            Exec(wszCmdLine, wszExeDir, TRUE);
          }

          if (*(wpCommandLineParams + 3))
          {
            int nTimeOut=10000;

            if (!hWndFireFox)
            {
              while (nTimeOut > 0 && !(hWndFireFox=FindFirefoxWindow()))
              {
                Sleep(100);
                nTimeOut-=100;
              }
            }

            if (nTimeOut > 0)
            {
              //Pass command line arguments
              wsprintfW(wszCmdLine, L"\"%s\" %s", wszTargetExe, wpCommandLineParams + 3);
              Exec(wszCmdLine, wszExeDir, FALSE);
            }
          }
        }
      }
    }
    CloseHandle(hMutex);
  }

  ExitProcess(0);
}

void Exec(wchar_t *wszCmdLine, wchar_t *wszExeDir, BOOL bWaitForIdle)
{
  STARTUPINFOW si;
  PROCESS_INFORMATION pi;

  xmemset(&si, 0, sizeof(STARTUPINFOW));
  si.cb=sizeof(STARTUPINFOW);
  GetStartupInfoW(&si);

  if (CreateProcessW(NULL, wszCmdLine, NULL, NULL, FALSE, 0, NULL, wszExeDir, &si, &pi))
  {
    if (bWaitForIdle)
      WaitForInputIdle(pi.hProcess, INFINITE);

    CloseHandle(pi.hProcess);
    CloseHandle(pi.hThread);
  }
}

HWND FindFirefoxWindow()
{
  HWND hWndFireFox=NULL;

  EnumWindows(EnumWindowProc, (LPARAM)&hWndFireFox);
  return hWndFireFox;
}

BOOL CALLBACK EnumWindowProc(HWND hWnd, LPARAM lParam)
{
  if (IsWindowVisible(hWnd) &&
      GetClassNameW(hWnd, wszWindow, MAX_PATH) && !lstrcmpiW(wszWindow, L"MozillaWindowClass") &&
      GetWindowTextW(hWnd, wszWindow, MAX_PATH) && xstrstrW(wszWindow, -1, L"Firefox", -1, FALSE, NULL, NULL))
  {
    *(HWND *)lParam=hWnd;
    return FALSE;
  }
  return TRUE;
}

DWORD GetFileSizeByName(const wchar_t *wpFile)
{
  HANDLE hFile;
  DWORD dwSize=0;

  hFile=CreateFileW(wpFile, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
  if (hFile != INVALID_HANDLE_VALUE)
  {
    dwSize=GetFileSize(hFile, NULL);
    CloseHandle(hFile);
  }
  return dwSize;
}

int GetExeDir(HINSTANCE hInstance, wchar_t *wszExeDir, int nLen)
{
  if (nLen=GetModuleFileNameW(hInstance, wszExeDir, nLen))
  {
    while (nLen > 0 && wszExeDir[nLen] != '\\') --nLen;
    wszExeDir[nLen]='\0';
  }
  return nLen;
}

wchar_t* GetCommandLineParamsW()
{
  wchar_t *wpCmdLine=GetCommandLineW();

  if (*wpCmdLine++ == '\"')
    while (*wpCmdLine != '\"' && *wpCmdLine != '\0') ++wpCmdLine;
  else
    while (*wpCmdLine != ' ' && *wpCmdLine != '\0') ++wpCmdLine;
  if (*wpCmdLine != '\0')
    while (*++wpCmdLine == ' ');

  return wpCmdLine;
}

BOOL WriteString(char *szFile, char *szStr, int nStrLen)
{
  HANDLE hFile;
  DWORD dwBytesWritten;

  if ((hFile=CreateFileA(szFile, GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN, NULL)) != INVALID_HANDLE_VALUE)
  {
    if (nStrLen == -1)
      nStrLen=lstrlenA(szStr);
    if (!WriteFile(hFile, szStr, nStrLen, &dwBytesWritten, NULL))
    {
      CloseHandle(hFile);
      return FALSE;
    }
    CloseHandle(hFile);
  }
  return TRUE;
}
